@@ -7,37 +7,27 @@
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "roommanager.h"
-#include "user.h"
+#include "spacehierarchycache.h"
+
+#include <Quotient/user.h>
 
 #include <QDebug>
+#if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
 #ifndef Q_OS_ANDROID
 #include <QDBusConnection>
 #include <QDBusInterface>
 #include <QDBusMessage>
 #endif
+#endif
 
 #include <KLocalizedString>
 #include <QGuiApplication>
 #include <utility>
 
-#ifndef QUOTIENT_07
-#include "notificationsmanager.h"
-#include <csapi/notifications.h>
-#endif
-
 using namespace Quotient;
 
 Q_DECLARE_METATYPE(Quotient::JoinState)
 
-#ifndef Q_OS_ANDROID
-bool useUnityCounter()
-{
-    static const auto Result = QDBusInterface("com.canonical.Unity", "/").isValid();
-
-    return Result;
-}
-#endif
-
 RoomListModel::RoomListModel(QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -46,34 +36,44 @@ RoomListModel::RoomListModel(QObject *parent)
         m_categoryVisibility[collapsedSection] = false;
     }
 
-#ifndef Q_OS_ANDROID
     connect(this, &RoomListModel::notificationCountChanged, this, [this]() {
-        if (useUnityCounter()) {
-            // copied from Telegram desktop
-            const auto launcherUrl = "application://org.kde.neochat.desktop";
-            // Gnome requires that count is a 64bit integer
-            const qint64 counterSlice = std::min(m_notificationCount, 9999);
-            QVariantMap dbusUnityProperties;
+#if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
+#ifndef Q_OS_ANDROID
+        // copied from Telegram desktop
+        const auto launcherUrl = "application://org.kde.neochat.desktop";
+        // Gnome requires that count is a 64bit integer
+        const qint64 counterSlice = std::min(m_notificationCount, 9999);
+        QVariantMap dbusUnityProperties;
 
-            if (counterSlice > 0) {
-                dbusUnityProperties["count"] = counterSlice;
-                dbusUnityProperties["count-visible"] = true;
-            } else {
-                dbusUnityProperties["count-visible"] = false;
-            }
-
-            auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat", "com.canonical.Unity.LauncherEntry", "Update");
-
-            signal.setArguments({launcherUrl, dbusUnityProperties});
-
-            QDBusConnection::sessionBus().send(signal);
+        if (counterSlice > 0) {
+            dbusUnityProperties["count"] = counterSlice;
+            dbusUnityProperties["count-visible"] = true;
+        } else {
+            dbusUnityProperties["count-visible"] = false;
         }
+
+        auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat", "com.canonical.Unity.LauncherEntry", "Update");
+
+        signal.setArguments({launcherUrl, dbusUnityProperties});
+
+        QDBusConnection::sessionBus().send(signal);
+#endif // Q_OS_ANDROID
+#else
+        qGuiApp->setBadgeNumber(m_notificationCount);
+#endif // QT_VERSION_CHECK(6, 6, 0)
+    });
+    connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceHierarchyChanged, this, [this]() {
+        Q_EMIT dataChanged(index(0, 0), index(rowCount(), 0), {IsChildSpaceRole});
     });
-#endif
 }
 
 RoomListModel::~RoomListModel() = default;
 
+Quotient::Connection *RoomListModel::connection() const
+{
+    return m_connection;
+}
+
 void RoomListModel::setConnection(Connection *connection)
 {
     if (connection == m_connection) {
@@ -154,10 +154,10 @@ void RoomListModel::doAddRoom(Room *r)
 void RoomListModel::connectRoomSignals(NeoChatRoom *room)
 {
     connect(room, &Room::displaynameChanged, this, [this, room] {
-        refresh(room);
+        refresh(room, {DisplayNameRole});
     });
-    connect(room, &Room::unreadMessagesChanged, this, [this, room] {
-        refresh(room);
+    connect(room, &Room::unreadStatsChanged, this, [this, room] {
+        refresh(room, {NotificationCountRole, HighlightCountRole});
     });
     connect(room, &Room::notificationCountChanged, this, [this, room] {
         refresh(room);
@@ -172,75 +172,18 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
         refresh(room);
     });
     connect(room, &Room::addedMessages, this, [this, room] {
-        refresh(room, {LastEventRole, SubtitleTextRole});
+        refresh(room, {LastEventRole, SubtitleTextRole, LastActiveTimeRole});
     });
     connect(room, &Room::pendingEventMerged, this, [this, room] {
         refresh(room, {LastEventRole, SubtitleTextRole});
     });
-#ifndef QUOTIENT_07
-    connect(room, &Room::notificationCountChanged, this, &RoomListModel::handleNotifications);
-#endif
-
-#ifndef QUOTIENT_07
-    connect(room, &Room::notificationCountChanged, this, &RoomListModel::refreshNotificationCount);
-#else
     connect(room, &Room::unreadStatsChanged, this, &RoomListModel::refreshNotificationCount);
-#endif
 }
 
-#ifndef QUOTIENT_07
-void RoomListModel::handleNotifications()
+int RoomListModel::notificationCount() const
 {
-    static bool initial = true;
-    static QStringList oldNotifications;
-    auto job = m_connection->callApi<GetNotificationsJob>();
-
-    connect(job, &BaseJob::success, this, [this, job]() {
-        const auto notifications = job->jsonData()["notifications"].toArray();
-        if (initial) {
-            initial = false;
-            for (const auto &n : notifications) {
-                oldNotifications += n.toObject()["event"].toObject()["event_id"].toString();
-            }
-            return;
-        }
-        for (const auto &n : notifications) {
-            const auto notification = n.toObject();
-            if (notification["read"].toBool()) {
-                oldNotifications.removeOne(notification["event"].toObject()["event_id"].toString());
-                continue;
-            }
-            if (oldNotifications.contains(notification["event"].toObject()["event_id"].toString())) {
-                continue;
-            }
-            oldNotifications += notification["event"].toObject()["event_id"].toString();
-
-            auto room = m_connection->room(notification["room_id"].toString());
-            auto currentRoom = RoomManager::instance().currentRoom();
-            bool roomIsActive = currentRoom && room->id() == currentRoom->id();
-
-            // If room exists, room is NOT active OR the application is NOT active, show notification
-            if (room && !(roomIsActive && QGuiApplication::applicationState() == Qt::ApplicationActive)) {
-                // The room might have been deleted (for example rejected invitation).
-                auto sender = room->user(notification["event"].toObject()["sender"].toString());
-
-                QImage avatar_image;
-                if (!sender->avatarUrl(room).isEmpty()) {
-                    avatar_image = sender->avatar(128, room);
-                } else {
-                    avatar_image = room->avatar(128);
-                }
-                NotificationsManager::instance().postNotification(dynamic_cast<NeoChatRoom *>(room),
-                                                                  sender->displayname(room),
-                                                                  notification["event"].toObject()["content"].toObject()["body"].toString(),
-                                                                  avatar_image,
-                                                                  notification["event"].toObject()["event_id"].toString(),
-                                                                  true);
-            }
-        }
-    });
+    return m_notificationCount;
 }
-#endif
 
 void RoomListModel::refreshNotificationCount()
 {
@@ -272,7 +215,7 @@ void RoomListModel::updateRoom(Room *room, Room *prev)
     }
     // Ok, we're through with pre-checks, now for the real thing.
     auto newRoom = static_cast<NeoChatRoom *>(room);
-    const auto it = std::find_if(m_rooms.begin(), m_rooms.end(), [=](const NeoChatRoom *r) {
+    const auto it = std::find_if(m_rooms.begin(), m_rooms.end(), [prev, newRoom](const NeoChatRoom *r) {
         return r == prev || r == newRoom;
     });
     if (it != m_rooms.end()) {
@@ -324,9 +267,6 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         return QVariant();
     }
     NeoChatRoom *room = m_rooms.at(index.row());
-    if (role == NameRole) {
-        return !room->name().isEmpty() ? room->name() : room->displayName();
-    }
     if (role == DisplayNameRole) {
         return room->displayName();
     }
@@ -365,9 +305,6 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         }
         return NeoChatRoomType::Normal;
     }
-    if (role == UnreadCountRole) {
-        return room->unreadCount();
-    }
     if (role == NotificationCountRole) {
         return room->notificationCount();
     }
@@ -401,12 +338,15 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     if (role == AvatarImageRole) {
         return room->avatar(128);
     }
-    if (role == IdRole) {
+    if (role == RoomIdRole) {
         return room->id();
     }
     if (role == IsSpaceRole) {
         return room->isSpace();
     }
+    if (role == IsChildSpaceRole) {
+        return SpaceHierarchyCache::instance().isChildSpace(room->id());
+    }
 
     return QVariant();
 }
@@ -425,13 +365,11 @@ void RoomListModel::refresh(NeoChatRoom *room, const QVector<int> &roles)
 QHash<int, QByteArray> RoomListModel::roleNames() const
 {
     QHash<int, QByteArray> roles;
-    roles[NameRole] = "name";
     roles[DisplayNameRole] = "displayName";
     roles[AvatarRole] = "avatar";
     roles[CanonicalAliasRole] = "canonicalAlias";
     roles[TopicRole] = "topic";
     roles[CategoryRole] = "category";
-    roles[UnreadCountRole] = "unreadCount";
     roles[NotificationCountRole] = "notificationCount";
     roles[HighlightCountRole] = "highlightCount";
     roles[LastEventRole] = "lastEvent";
@@ -441,44 +379,45 @@ QHash<int, QByteArray> RoomListModel::roleNames() const
     roles[CategoryVisibleRole] = "categoryVisible";
     roles[SubtitleTextRole] = "subtitleText";
     roles[IsSpaceRole] = "isSpace";
-    roles[IdRole] = "id";
+    roles[RoomIdRole] = "roomId";
+    roles[IsChildSpaceRole] = "isChildSpace";
     return roles;
 }
 
-QString RoomListModel::categoryName(int section)
+QString RoomListModel::categoryName(int category)
 {
-    switch (section) {
-    case 1:
+    switch (category) {
+    case NeoChatRoomType::Invited:
         return i18n("Invited");
-    case 2:
+    case NeoChatRoomType::Favorite:
         return i18n("Favorite");
-    case 3:
+    case NeoChatRoomType::Direct:
         return i18n("Direct Messages");
-    case 4:
+    case NeoChatRoomType::Normal:
         return i18n("Normal");
-    case 5:
+    case NeoChatRoomType::Deprioritized:
         return i18n("Low priority");
-    case 6:
+    case NeoChatRoomType::Space:
         return i18n("Spaces");
     default:
         return "Deadbeef";
     }
 }
 
-QString RoomListModel::categoryIconName(int section)
+QString RoomListModel::categoryIconName(int category)
 {
-    switch (section) {
-    case 1:
+    switch (category) {
+    case NeoChatRoomType::Invited:
         return QStringLiteral("user-invisible");
-    case 2:
+    case NeoChatRoomType::Favorite:
         return QStringLiteral("favorite");
-    case 3:
+    case NeoChatRoomType::Direct:
         return QStringLiteral("dialog-messages");
-    case 4:
+    case NeoChatRoomType::Normal:
         return QStringLiteral("group");
-    case 5:
+    case NeoChatRoomType::Deprioritized:
         return QStringLiteral("object-order-lower");
-    case 6:
+    case NeoChatRoomType::Space:
         return QStringLiteral("group");
     default:
         return QStringLiteral("tools-report-bug");
@@ -516,7 +455,9 @@ NeoChatRoom *RoomListModel::roomByAliasOrId(const QString &aliasOrId)
     return nullptr;
 }
 
-int RoomListModel::indexForRoom(NeoChatRoom *room) const
+int RoomListModel::rowForRoom(NeoChatRoom *room) const
 {
     return m_rooms.indexOf(room);
 }
+
+#include "moc_roomlistmodel.cpp"
