@@ -3,6 +3,7 @@
 // SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 #include "roommanager.h"
+
 #include "controller.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
@@ -10,12 +11,11 @@
 #include <QDesktopServices>
 #include <QQuickTextDocument>
 #include <QStandardPaths>
-#include <csapi/joining.h>
-#ifdef QUOTIENT_07
-#include <csapi/knocking.h>
-#endif
-#include <qt_connection_util.h>
-#include <user.h>
+
+#include <Quotient/csapi/joining.h>
+#include <Quotient/csapi/knocking.h>
+#include <Quotient/qt_connection_util.h>
+#include <Quotient/user.h>
 
 #ifndef Q_OS_ANDROID
 #include <KIO/OpenUrlJob>
@@ -121,8 +121,6 @@ void RoomManager::openRoomForActiveConnection()
         if (room) {
             enterRoom(room);
         }
-    } else {
-        Q_EMIT pushWelcomePage();
     }
 }
 
@@ -133,9 +131,13 @@ void RoomManager::enterRoom(NeoChatRoom *room)
     }
     if (m_currentRoom && m_chatDocumentHandler) {
         // We're doing these things here because it is critical that they are switched at the same time
-        m_currentRoom->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
-        m_chatDocumentHandler->setRoom(room);
-        m_chatDocumentHandler->document()->textDocument()->setPlainText(room->savedText());
+        if (m_chatDocumentHandler->document()) {
+            m_currentRoom->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
+            m_chatDocumentHandler->setRoom(room);
+            m_chatDocumentHandler->document()->textDocument()->setPlainText(room->savedText());
+        } else {
+            m_chatDocumentHandler->setRoom(room);
+        }
     }
     m_lastCurrentRoom = std::exchange(m_currentRoom, room);
     Q_EMIT currentRoomChanged();
@@ -146,6 +148,10 @@ void RoomManager::enterRoom(NeoChatRoom *room)
         Q_EMIT replaceRoom(m_currentRoom, QString());
     }
 
+    if (room && room->timelineSize() == 0) {
+        room->getPreviousContent(20);
+    }
+
     // Save last open room
     m_lastRoomConfig.writeEntry(Controller::instance().activeConnection()->userId(), room->id());
 }
@@ -160,16 +166,12 @@ UriResolveResult RoomManager::visitUser(User *user, const QString &action)
 {
     if (action == "mention" || action.isEmpty()) {
         // send it has QVariantMap because the properties in the
-#ifdef QUOTIENT_07
         user->load();
-#endif
         Q_EMIT showUserDetail(user);
     } else if (action == "_interactive") {
         user->requestDirectChat();
     } else if (action == "chat") {
-#ifdef QUOTIENT_07
         user->load();
-#endif
         Q_EMIT askDirectChatConfirmation(user);
     } else {
         return Quotient::IncorrectAction;
@@ -212,8 +214,6 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
     });
 }
 
-// TODO: maybe need use the function upstream later
-#ifdef QUOTIENT_07
 void RoomManager::knockRoom(Quotient::Connection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers)
 {
     auto *const job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, reason);
@@ -231,7 +231,6 @@ void RoomManager::knockRoom(Quotient::Connection *account, const QString &roomAl
         }
     });
 }
-#endif
 
 bool RoomManager::visitNonMatrix(const QUrl &url)
 {
@@ -285,3 +284,5 @@ void RoomManager::setChatDocumentHandler(ChatDocumentHandler *handler)
     m_chatDocumentHandler->setRoom(m_currentRoom);
     Q_EMIT chatDocumentHandlerChanged();
 }
+
+#include "moc_roommanager.cpp"
